package moca.core.proxy;

import moca.core.proxy.communication.CommunicationProtocol;
import moca.core.proxy.receivers.Package;
import moca.core.proxy.state.Configurator;
import moca.core.proxy.state.State;
import moca.core.proxy.state.StateConf;
import moca.service.cis.util.DeviceContext;

/**
 * Contem informaes sobre o cliente, destinatario da mensagem
 * @author hana
 */
public class ClientInfo {
	/**
	* Identificador unico do cliente
	*/
	private String clientId;
	
	/**
	 * The communication protocol used by the client to send/receive messages to the proxy
	 */
	private CommunicationProtocol protocol;
	/**
	* estados monitorados
	*/
	private State[] states;
	
	//Hana - new 01/08/06
	/**
	 *  Contexto atual do dispositivo, com os valores correntes de cada varivel de contexto
	 */
	private DeviceContext  currentDeviceContext; // Lembrar que este valor pode diferir do valor que ativou uma adaptao, j que no momento do processamento da mesma 
	                                        //um novo contexto pode ter chegado (thread diferente) ...  Deve ser usado mais para contextos estticos	
	/**
	 * Ultimo pacote enviado para este cliente
	 */
	private Package lastPackage;
	/**
	 * Retorna o ultimo pacote enviado para este cliente. 
	 * Obs: Este metodo  utilizado pelo
	 * proxy para manter a ordem de envio dos pacotes
	 * @return ultimo pacote 
	 */
	public final Package getLastPackage() {
		return lastPackage;
	}
	/**
	 * altera o ultimo pacote destinado ao usuario. Obs: Este metodo
	 *  utilizado pelo proxy para manter a ordem de envio dos pacotes
	 * @param pack
	 */
	public final void setLastPackage(Package pack) {
		this.lastPackage = pack;
	}
	/**
	 * Construtor que recebe o identificador do cliente
	 * @param clientId  Identificador do cliente
	 * @param protocol  Protocolo de comunicao usado pelo cliente
	 */
	public ClientInfo(String clientId, CommunicationProtocol prot) {
		this.clientId = clientId;
		this.protocol = prot;
		Configurator conf = Configurator.getInstance();
		StateConf[] stateConfs = conf.getStateConfArray();
		states = new State[stateConfs.length];
		for (int i = 0; i < stateConfs.length; i++) {
			states[i] = new State(stateConfs[i]);
		}
	}
	/**
	 * Identificador do cliente
	 * @return identificador do cliente
	 */
	public String getClientId() {
		return this.clientId;
	}
	/**
	 * Retorna a classe representativa do 
	 * estado monitorado para o cliente. Cada estado
	 * possui um ndice, atribuido de acordo com a ordem de declarao no XML
	 * @param index indice do estado
	 * @return classe que representa o estado do cliente
	 */
	public State getState(int index) {
		return states[index];
	}
	/**
	 * Retorna o numero de estados monitorados para o cliente
	 * @return numero de estados monitorados
	 */
	public int getNumStates() {
		return states.length;
	}
	/**
	*  Apresenta informacoes relevantes do cliente para um string
	*  @return string
	*/
	public String toString() {
		return this.clientId;
	}
		
	/**
	 * @return Returns the protocol.
	 */
	public CommunicationProtocol getCommunicationProtocol() {
		return protocol;
	}
	/**
	 * @param protocol The protocol to set.
	 */
	public void setCommuniationProtocol(CommunicationProtocol protocol) {
		this.protocol = protocol;
	}
	
	// Hana - new - 01/08/06
	/**
	 * @return Returns current (device) context of this client
	 */
	public DeviceContext getCurrentContext(){
		return this.currentDeviceContext;
	}
	/**
	 * 
	 * @param currentContext  The current device contexts obtained from CIS
	 */
	public void setCurrentContext(DeviceContext currentContext){
		this.currentDeviceContext = currentContext;
	}
}
