package moca.core.proxy.actions;

import java.util.HashMap;
import java.util.Map;

/**
 *   a classe base para aes customizadas sobre mensagens de acordo com o estado
 *  do dispositivo destinatrio da mensagem
 * 
 * @author hana
 *
 */
public abstract class Action {
	/**
	* Mapa onde serao armazenados os parametros fornecidos  ao 
	*/
	private Map parameters;
	/**
	 * Permite a recuperao do valor de um parametro de acordo com seu nome
	 * @param paramName
	 * @return
	 */
	protected final Object getParameter(String paramName) {
		//Se o parametro mapa de parametros for nulo, retorna null
		if (parameters == null) {
			return null;
		}
		//retorna o valor do parametro associado ao nome 'paramName'
		return parameters.get(paramName);
	}
	/**
	 * Inicializa o mapa de parametros. A cada parametro esta associado um nome
	 * para sua posterior recuperao
	 * @param map mapa contendo entradas do tipo 
	 * 						(nome do parametro, valor do parametro)
	 */
	public void init(Map map) {
		this.parameters = new HashMap(map);
	}
	/**
	* Compara dois objetos
	*/
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}
		if (obj.getClass() != this.getClass()) {
			return false;
		}
		Action action = (Action) obj;
		if (this.parameters.equals(action.parameters)) {
			return true;
		}
		return false;
	}

}
