/*
 * Created on 08/11/2004
 *
 */
package moca.core.proxy.communication;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.SocketException;
import java.net.UnknownHostException;

import moca.core.proxy.ProxyException;
import moca.core.proxy.communication.async.ClientSubscriptionListener;
import moca.core.proxy.communication.async.MyEventProvider;
import moca.core.proxy.communication.sync.SynchronousTCPServer;
import moca.core.proxy.communication.sync.SynchronousUDPServer;
import moca.core.proxy.development.MessageListener;
import moca.core.proxy.development.SubscribeListener;
import moca.core.proxy.message.Message;
import moca.core.proxy.util.ProxyConfiguration;
import moca.protocol.net.async.DataListener;
import moca.protocol.net.async.ReceivedData;
import moca.protocol.net.async.tcp.AsyncTCPServer;
import moca.protocol.net.sync.tcp.SyncTCPServer;
import moca.protocol.net.sync.udp.UDPDatagram;
import moca.util.eci.ECIConstants;
import moca.util.eci.Subscription;

import org.apache.log4j.Logger;

/**
 * Perform all communication between the proxy and the clients.
 * It supports synchronous and asynchronous communication, with TCP or UDP protocols.
 * Provide methods to send(or publish) and receive messages to/from mobile clients.
 * 
 * @author hana
 */
public class ClientCommunication {

	//static Category 			cat = Category.getInstance(ClientCommunication.class.getName());
	private static final Logger cat = Logger.getLogger(ClientCommunication.class);
	
	// o end 0.0.0.0  uma mcara para pegar todos os IPs da mquina...
	// assim a parte servidora (comunicao com cliente) pode aceitar requisies 
	// em qquer uma das interfaces de rede disponveis
   public static String     DEFAULT_SERVER_IP = "0.0.0.0";
	public static int        DEFAULT_SERVER_PORT = 55100;
	public static int        DEFAULT_PUBLISHER_PORT = 55110;

	// configurao dos endereos e portas do proxy
	//int      syncServerPort;
	int      serverUDPPort ;
	int      eventServerPort;
	int      clientPort = 0;
	String   serverLocalAddress; // pode tvz ouvir em vrias interfaces (no caso de 0.0.0.0)

	// comunicao lado cliente
	/** Asynchronous server:  pub/sub only for subjects (no expression) -  answers TCP and UDP */
	private  MyEventProvider 		eventServer;	
	
	/** Synchronous server: TCP and UDP */
	private	UDPDatagram			udpServer = null;
	private  SyncTCPServer		tcpServer = null;
	private AsyncTCPServer connection = null ;
	private SynchronousTCPServer tcpServerThread = null ;
	SynchronousUDPServer udpServerThread = null ;
	// Por enquanto  s um... precisaria de mais???
	/** Listener for client messages */
	private MessageListener msgListener=null;
	
	
	// TODO Por enquanto  s um... precisaria de mais???
	private SubscribeListener subscListener = null;
	
	
	/**
	 *  Constructor. Defines the addresses to create servers that wait for client requests
	 */
	public ClientCommunication() {
		//syncServerPort = DEFAULT_SERVER_PORT;
		eventServerPort = DEFAULT_PUBLISHER_PORT;
		serverLocalAddress = DEFAULT_SERVER_IP;
		
		configureAddresses();
	}
	

	/**
	 * Try to get the proxy's adresses configuration from the properties file.
	 * Get the IP addresses and ports for synchronous and asynchronous communication. 
	 */
	private void configureAddresses(){
		
		try{
			String _serverLocalAddr  = ProxyConfiguration.getString("proxy.server.ip_address"); //$NON-NLS-1$
			//String _serverPort = ProxyConfiguration.getString("proxy.sync_server.port"); //$NON-NLS-1$
			String _evServerPort = ProxyConfiguration.getString("proxy.event_server.port"); //$NON-NLS-1$
			String _clientPort = ProxyConfiguration.getString("proxy.server_tcp.port");	
			String _SeverUdpPort = ProxyConfiguration.getString("proxy.server_udp.port");
			
			
			clientPort = Integer.parseInt(_clientPort) ;
		//	syncServerPort = Integer.parseInt(_serverPort);
			eventServerPort = Integer.parseInt(_evServerPort);
			serverLocalAddress = _serverLocalAddr;
			serverUDPPort = Integer.parseInt(_SeverUdpPort);
		}
		catch(Throwable e) {
         cat.error("There was some error when reading proxy.properties file!",e);
         System.out.println("There was some error when reading proxy.properties file." +
         		" \nThe default server configuration will be used.\n");
		}
			         
	//	cat.info(" Proxy Address --> Server at " + serverLocalAddress + " with async port: " +
	//			eventServerPort + " and sync port: " + syncServerPort );				
	}

	/**
	 * Create and start an asynchronous server to answer clients subscriptions.
	 * Based on ECI (Even-based Communication Interface)
	 */	
	public void startEventServer(String protocol)
	{	
		// Criar servidor assincrono para comunicao com os clientes moveis		
		InetSocketAddress asyncServerAddress = new InetSocketAddress(serverLocalAddress, eventServerPort);    
		
		
		try
		{
			//TODO POR ENQUANTO este EventProvider s atende a subscries por assunto,
			// SEM expresses de interesse!!
			if(protocol.equals(CommunicationProtocol.TCP))
			    eventServer = new MyEventProvider(asyncServerAddress,false,this, ECIConstants.TCP );
			else if(protocol.equals(CommunicationProtocol.UDP))
			    eventServer = new MyEventProvider(asyncServerAddress,false,this, ECIConstants.UDP );
			else{
				throw new IOException("wrong type of protocol. It must be CommunicationProtocol.TCP or CommunicationProtocol.UDP");
			}
			// quando receber uma subscrio, repass-la para a aplicao 
			eventServer.addSubscriptionListener(new ClientSubscriptionListener(this));
		
			
		}
		catch (IOException e2)
		{
			cat.fatal("It was not possible to create one instance of the Proxy's eventServer",e2); //$NON-NLS-1$
		    System.exit(1);
		}
			
	}
	
	public void startAsynchronousTCPServer(){
		InetSocketAddress localAddress = new InetSocketAddress(serverLocalAddress, clientPort);
		   
    	try {
    		connection = new AsyncTCPServer(localAddress) ;
			connection.addListener(new TCPAsyncListener(connection)) ;
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	
	private class TCPAsyncListener implements DataListener{
		AsyncTCPServer connection = null ;
		public TCPAsyncListener(AsyncTCPServer connection){
			 this.connection = connection ;
		}
		public void onReceiveData(ReceivedData arg0) {
			CommunicationProtocol com = null;
			Message msg = (Message)arg0.getData() ;
			try {
				com = new CommunicationProtocol(CommunicationProtocol.TCP,CommunicationProtocol.ASYNC);
			    com.setAddress( ((Message)arg0.getData()).getClientAddress()  ); 
				com.setAsyncTCPConnection(connection) ;
			   // com.setAddress(msg.getClientAddress() );
			
			} catch (ProxyException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
			notifyReceivedMessage(msg, com);
			
		}
		
	}
	/**
	 * Create a server to answer TCP requests from mobile clients
	 */
	public void startSynchronousTCPServer()
	{
		InetSocketAddress proxyLocalAddress;
		
		try
		{
			proxyLocalAddress = new InetSocketAddress(serverLocalAddress, clientPort);

			tcpServer = new SyncTCPServer(proxyLocalAddress);
			
			tcpServerThread = new SynchronousTCPServer(tcpServer,this);
			tcpServerThread.start();
		}
		catch (UnknownHostException e)
		{
			cat.error("It was not possible to resolver Proxy's name", e); //$NON-NLS-1$
		}
		catch (IOException e2)
		{
			cat.error("It was not possible create an instance of the Synchronous TCP Server",e2); //$NON-NLS-1$
		}
		
	}
	

	/**
	 * Create a server to answer UDP requests from mobile clients
	 */
	public void startSynchronousUDPServer()
	{
		InetSocketAddress proxyLocalAddress;
		
		try
		{
			proxyLocalAddress = new InetSocketAddress(serverLocalAddress, serverUDPPort);
			
			udpServer = new UDPDatagram(proxyLocalAddress);
		    udpServerThread = new SynchronousUDPServer(udpServer,this);
			udpServerThread.start();

		}
		catch (SocketException e1)
		{
			cat.error("It was not possible create an instance of the Synchronous UDP Server",e1); //$NON-NLS-1$
		}

	}
		
	/**
	 * @return Returns the eventServer.
	 */
	public MyEventProvider getEventServer() {
		return eventServer;
	}
	/**
	 * @return Returns the tcpServer.
	 */
	public SyncTCPServer getTcpServer() {
		return tcpServer;
	}
	/**
	 * @return Returns the udpServer.
	 */
	public UDPDatagram getUdpServer() {
		return udpServer;
	}
	
	
	// Por enquanto s tem um listener de cada tipo.. mas tvz tenha
	// que ter uma lista
	/**
	 *  Register the listener for received messages
	 * @param msgListener The msgListener to set.
	 */
	public void addMessageListener(MessageListener msgListener) {
		this.msgListener = msgListener;
	}
	
	/**
	 * Repass for the aplication the message received from the client
	 * @param msg  The Message
	 * @param protocol Client communication protocol. Note that if the message is
	 *                 from a pub/sub, this protocol only contains the communication mode = ASYNC.
	 */
	public void notifyReceivedMessage(Message msg, CommunicationProtocol protocol ){
		//	TODO: Qdo tiver mais de um: Varrer os listeners para encaminhar a mensagem
		msgListener.onReceiveMessage(msg,protocol);
	}
	
	
	/**
	 * Repass for the application the message received from the client,
	 * via asynchronous communication 
	 * @param subject The subject of the published message
	 * @param data the message, must be an Message object
	 */
	public void notifyPublishedMessage(String subject, Object data, Integer protocol){
		Message msg = (Message) data;
		msg.setSubject(subject);
		
		// Como foi uma publicao, no se sabe o cliente que publicou, apenas
		// para qual assunto deva ser publicado
		// Assim o Communication Protocol, ter apenas o tipo da comunicao,
		// para indicar isso.
		
		CommunicationProtocol cm = null;
		
		try {
			//	 Enviar a msg para a aplicao, junto com o protocolo
			if(protocol.equals(ECIConstants.TCP))
			    cm = new CommunicationProtocol(CommunicationProtocol.TCP,CommunicationProtocol.EVENT);			
			else if(protocol.equals(ECIConstants.UDP))
				cm = new CommunicationProtocol(CommunicationProtocol.UDP,CommunicationProtocol.EVENT);	
		} catch (ProxyException e) {
			cat.error("Error during cration of communicationProtocol");
		}		
		
		notifyReceivedMessage(msg, cm);
	}
	
	/**
	 *  Register the listener to inform the subscription of a client
	 * @param subscListener The subscListener to set.
	 */
	public void addSubscribeListener(SubscribeListener subscListener) {
		this.subscListener = subscListener;
	}
	
	/**
	 * @param subscription The subscription (ou usubscribe) made by the cliente. Contain the subject of interest.
	 * @param cm  Client communication protocol, i.e, its address 
	 *            (note that the client identification is not available, and after
	 *             the application will have to provide it to the proxy) 
	 */
	public void notifySubscriptionToApp(Subscription subscription, CommunicationProtocol cm){
		//	TODO: Qdo tiver mais de um: Varrer os listeners para encaminhar a mensagem
		subscListener.onReceiveSubscription(subscription,cm); 
	}
	
	public void freeResources(){
		try {
			if(udpServerThread != null){
				udpServerThread.freeResources() ;
			}
			if(connection != null){
				connection.removeAllListener() ;
			    connection.freeResources() ;
			}
			if(tcpServer != null){
			     tcpServerThread.freeResources();
			}
			
			if(eventServer != null){
				eventServer.removeAllSubscriptionListeners();
				eventServer.freeResources() ;
			}
			
			if(udpServer != null){
				udpServer.freeResources() ;
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		connection = null ;
		//tcpServerThread = null ;
	}
}
