/*
 * Created on 13/10/2005
 */
package moca.core.proxy.communication.sync;


import java.io.IOException;

import java.net.DatagramPacket;
import java.net.SocketException;

import java.net.InetSocketAddress;

import org.apache.log4j.Category;

import moca.core.proxy.ProxyException;
import moca.core.proxy.communication.CommunicationProtocol;
import moca.core.proxy.communication.ServerCommunication;
import moca.core.proxy.message.Message;
import moca.core.proxy.util.ProxyConfiguration;
import moca.protocol.net.sync.udp.UDPDatagram;


/**
 * This sample shows how to create a client application for communicating with
 * a synchronous UDP server (UDPDatagram). It prompts the user for typing a 
 * message to be sent to the server. The server processes the message and returns
 * a reply.
 * 
 * @author Fabio B. Pereira 
 */
public class SynchronousUDPClient extends Thread
{
    
    private	UDPDatagram 		udpClient		= null;
    private InetSocketAddress   serverAddress   = null; 
    private ServerCommunication serverComm = null;
    private Message				reply			= null;
    static   Category			cat = Category.getInstance(SynchronousUDPClient.class.getName());
    boolean running = false ;
    public SynchronousUDPClient(InetSocketAddress serverAddress, ServerCommunication sc) 
    {		
		this.serverAddress = serverAddress ;
		serverComm = sc ;
    }
    
    public void run()
    {   
        
            try {
            	String _evServerPort = ProxyConfiguration.getString("proxy.client_udp.port");
				this.udpClient = new UDPDatagram(new InetSocketAddress("0.0.0.0",Integer.parseInt( _evServerPort)));
				
			} catch (SocketException e1) {
				cat.error("It was no possible to create UDP datagram socket",e1) ;
			}
            running =true ; 
            while(running)
	        {
               CommunicationProtocol cm;
               
	           reply = getServerReply();
	           
	           if(reply == null)
	        	   continue ;
	           
			   try {
				
				   cm = new CommunicationProtocol(CommunicationProtocol.UDP,CommunicationProtocol.SYNC);
				   cm.setAddress(serverAddress);
				   serverComm.notifyReceivedMessage(reply,cm);
			   } catch (ProxyException e) {
				   cat.error("Erro Intancing CommunicationProtocol, check parameters",e) ;
			   }
				
	        }// while
    }

    public void sendMsg(Message m) throws IOException{
    	   byte [] buffer = UDPDatagram.convertObjectToByteArray(m);
			udpClient.send(buffer ,serverAddress);
		
    }
    
    private Message getServerReply()
	{
	    DatagramPacket 	data  = null;
	    Message	ret = null ;

		try
		{
		    // Waits for a reply from the server for a period of time
			data = udpClient.receive() ;
			
			// If there is received data
			if(data != null)
			{
			    // Converts the received array of bytes into an Reply object
				if(data.getData() != null ){
			       ret =  (Message)UDPDatagram.convertBytesArrayToObject(data.getData());
				}
			   
			}
			
		}
		catch (ClassNotFoundException e) {
			cat.error("Error in getServerReply() - ClassNotFoundException. ",e);
		} 
		catch (IOException e) {
		    cat.warn("Error in getServerReply() - IOException. It was not possible to receive the reply from the Server",e);
		    udpClient.closeSocket() ;
		}
		
		return ret ;
	}
    
    public void freeResources(){
    	running = false ;
    	
    	
    	if(udpClient != null){
			try {
				udpClient.freeResources() ;
			} catch (IOException e) {}
    	}
    }

}
