package moca.core.proxy.state.filter;

/**
 * Verifique se o operando 1 inicia com o operando 2. A comparao  case
 * insensitive
 * @author hana
 *
 */
public class StartWithOperator extends BinaryOperator implements BooleanValue {
	/**
	 * Construtor que recebe dois operandos do tipo StringValue
	 * @param sv1 operando 1
	 * @param sv2 operando 2
	 */
	public StartWithOperator(StringValue sv1, StringValue sv2) {
		super(sv1, sv2);
	}
	/**
	 * Valora o operador. Verifica se o operando 1 inicia com o operando
	 * 2. A comparacao  case insensitive
	 * @param metapack
	 * @return true se inicia e false caso contrario
	 */
	public Object evaluate(MetaPackage metapack) {
		String s1 = ((StringValue) this.val1).getStringValue(metapack);
		String s2 = ((StringValue) this.val2).getStringValue(metapack);
		if (s1 == null || s2 == null) {
			return new Boolean(false);
		}
		return new Boolean(s1.toLowerCase().startsWith(s2.toLowerCase()));
	}
	/**
	 * Valora segunda a interface BooleanValue
	 * @param metapack
	 * @return value
	 */
	public boolean getBooleanValue(MetaPackage metapack) {
		Boolean b = (Boolean) this.evaluate(metapack);
		return b.booleanValue();
	}
	/**
	 * Converte a expressao para uma string
	 * @return expressao na forma de string
	 */
	public String toString() {
		return "(" + this.val1 + ".StarWith(" + this.val2 + "))";
	}


}
